local addonName, addon = ...

local L = addon.language
local config = addon.config
local frame = addon.frame
local avoidableDamage = addon.avoidableDamage

frame:SetScript('OnEvent', function(self, event, ...)
	local a = self[event]
	if a then
		a(self, ...)
	end
end)

frame:RegisterEvent('ADDON_LOADED')
frame:RegisterEvent('PLAYER_LOGIN')
frame:RegisterEvent('PLAYER_LOGOUT')
frame:RegisterEvent("MERCHANT_SHOW")
frame:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
frame:RegisterEvent("CHALLENGE_MODE_START")
frame:RegisterEvent("CHALLENGE_MODE_COMPLETED")

function frame:ADDON_LOADED(name)
	if name ~= addonName then return end
	self:UnregisterEvent('ADDON_LOADED')
	if not AkiDB then AkiDB = {} end
	addon:UpdateConfig(config, AkiDB)
end

function frame:PLAYER_LOGOUT()
	addon:UpdateConfig(AkiDB, config)
end

function frame:MERCHANT_SHOW( ) 
	local num = 0
	if config.autoSell then
		for bag = 0, NUM_BAG_FRAMES do
			for slot = 1, GetContainerNumSlots(bag) do
				local itemId = GetContainerItemID(bag,slot)
				if itemId then 
					local _, _, itemRarity = GetItemInfo(itemId)
					if itemRarity == 0 then
						UseContainerItem(bag,slot)
						num = num +1
					end 
				end 
			end
		end
		if (num > 0) then
			print(L["autoSell"].."["..num.."]")
		end
	end
end

function frame:COMBAT_LOG_EVENT_UNFILTERED( ...) 
    local timestamp, eventType, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags = CombatLogGetCurrentEventInfo()

	local inInstance = IsInGroup(LE_PARTY_CATEGORY_INSTANCE)
	local inParty = IsInGroup()
	local inRaid = IsInRaid()
	local cfg
	if inInstance then
		cfg = config.instance
	elseif inRaid then
		cfg = config.raid
	elseif inParty and not inRaid  then
		cfg = config.party
	else
		cfg = config.solo
	end

	if eventType=="SPELL_INTERRUPT" then
		local spellId, spellName, spellSchool, extraSpellId, extraSpellName, extraSchool = select(12, CombatLogGetCurrentEventInfo())
		if sourceName==UnitName("player")  or sourceName==UnitName("pet")then
			addon:Announce(cfg.ownInterrupt, L.interrupt.." → "..GetSpellLink(extraSpellId), config.soundOwn)
		elseif addon:IsInMyGroup(sourceFlags) then
			local name = addon:ShortName(sourceName)
			if cfg.other == 'self' and addon:IsPlayer(sourceFlags) then
				name = addon:ClassColor(name)
			end
			addon:Announce(cfg.otherInterrupt, "["..name.."]"..L["interrupt"].." → "..GetSpellLink(extraSpellId), config.soundOther)
		end

	elseif eventType=="SPELL_DISPEL" then
		local spellId, spellName, spellSchool, extraSpellId, extraSpellName, extraSchool, auraType = select(12, CombatLogGetCurrentEventInfo())
		if sourceName==UnitName("player")  or sourceName==UnitName("pet")then
			addon:Announce(cfg.ownDispel, L["dispel"].." → "..GetSpellLink(extraSpellId), "")
		elseif addon:IsInMyGroup(sourceFlags) then
			local name = addon:ShortName(sourceName)
			if cfg.other == 'self' and addon:IsPlayer(sourceFlags) then
				name = addon:ClassColor(name)
			end
			addon:Announce(cfg.otherDispel, "["..name.."]"..L["dispel"].." → "..GetSpellLink(extraSpellId), "")
		end

	elseif eventType=="SPELL_STOLEN" then
		local spellId, spellName, spellSchool, extraSpellId, extraSpellName, extraSchool, auraType = select(12, CombatLogGetCurrentEventInfo())
		if sourceName==UnitName("player") then
			addon:Announce(cfg.ownStolen, L["stolen"].." → "..GetSpellLink(extraSpellId), "")
		elseif addon:IsInMyGroup(sourceFlags) then
			local name = addon:ShortName(sourceName)
			if cfg.other == 'self' and addon:IsPlayer(sourceFlags) then
				name = addon:ClassColor(name)
			end
			addon:Announce(cfg.otherStolen, "["..name.."]"..L["stolen"].." → "..GetSpellLink(extraSpellId), "")
		end

	elseif eventType=="SPELL_MISSED" then
		local spellId, spellName, spellSchool, missType, isOffHand, amountMissed = select(12, CombatLogGetCurrentEventInfo())
		if missType=="REFLECT" and destName==UnitName("player") then 
			addon:Announce(cfg.ownReflect, L["reflect"].." → "..GetSpellLink(spellId), "")
		elseif missType=="REFLECT" and addon:IsInMyGroup(sourceFlags) then
			local name = addon:ShortName(sourceName)
			if cfg.other == 'self' and addon:IsPlayer(sourceFlags) then
				name = addon:ClassColor(name)
			end
			addon:Announce(cfg.otherReflect, "["..name.."]"..L["reflect"].." → "..GetSpellLink(spellId), "")
		end

	elseif eventType == "SPELL_DAMAGE" or eventType == "SPELL_PERIODIC_DAMAGE" or eventType == "SPELL_BUILDING_DAMAGE" or eventType == "RANGE_DAMAGE" then
		local spellId, spellName, spellSchool, amount = select(12,CombatLogGetCurrentEventInfo())
		if destName == UnitName("player") then
			addon:SpellDamage(cfg.ownAD, destName, spellId, amount)
		elseif addon:IsInMyGroup(destFlags) then
			local name = addon:ShortName(destName)
			if cfg.other == 'self' and addon:IsPlayer(destFlags) then
				name = addon:ClassColor(name)
			end
			addon:SpellDamage(cfg.otherAD, name, spellId, amount)
		end

	end
end

frame:RegisterEvent("CHAT_MSG_BN_WHISPER")
function frame:CHAT_MSG_BN_WHISPER( ... ) 
	local msg = select(1, ...)
	local bnetIDAccount = select(13, ...)
	local characterName , bnetIDGameAccount = select(5, BNGetFriendInfoByID(bnetIDAccount))
	if msg == config.inviteMsg then
		print(L["inviting"].."-->"..characterName)
		BNInviteFriend(bnetIDGameAccount)
	end
end

frame:RegisterEvent("CHAT_MSG_WHISPER")
function frame:CHAT_MSG_WHISPER( ... ) 
	local msg, charname = ...
	print(msg.." - "..charname)
	if msg == config.inviteMsg then
		print(L["inviting"].."-->"..charname)
		InviteUnit(charname)
	end
end

function frame:CHALLENGE_MODE_COMPLETED(...)
	addon:AnnounceAD("self")
end

function frame:CHALLENGE_MODE_START(...)
	avoidableDamage["cache"] = {}
	avoidableDamage["sum"] = {}
	avoidableDamage["saveByName"] = {}
	avoidableDamage["saveBySpell"] = {}
	avoidableDamage["timer"] = {}
	avoidableDamage["list"] = {}
	avoidableDamage["alist"] = {}
end